@@ -2,16 +2,13 @@
 Module providing easy API for working with remote files and folders.
 """
 
-from __future__ import with_statement
-
 import hashlib
-import tempfile
-import re
 import os
-from StringIO import StringIO
+import six
+
 from functools import partial
 
-from fabric.api import *
+from fabric.api import run, sudo, hide, settings, env, put, abort
 from fabric.utils import apply_lcwd
 
 
@@ -25,9 +22,13 @@ def exists(path, use_sudo=False, verbose=False):
     stderr and any warning resulting from the file not existing) in order to
     avoid cluttering output. You may specify ``verbose=True`` to change this
     behavior.
+
+    .. versionchanged:: 1.13
+        Replaced internal use of ``test -e`` with ``stat`` for improved remote
+        cross-platform (e.g. Windows) compatibility.
     """
     func = use_sudo and sudo or run
-    cmd = 'test -e %s' % _expand_path(path)
+    cmd = 'stat %s' % _expand_path(path)
     # If verbose, run normally
     if verbose:
         with settings(warn_only=True):
@@ -43,13 +44,14 @@ def is_link(path, use_sudo=False, verbose=False):
 
     If ``use_sudo`` is True, will use `.sudo` instead of `.run`.
 
-    `.is_link` will, by default, hide all output. Give ``verbose=True`` to change this.
+    `.is_link` will, by default, hide all output. Give ``verbose=True`` to
+    change this.
     """
     func = sudo if use_sudo else run
     cmd = 'test -L "$(echo %s)"' % path
     args, kwargs = [], {'warn_only': True}
     if not verbose:
-        opts = [hide('everything')]
+        args = [hide('everything')]
     with settings(*args, **kwargs):
         return func(cmd).succeeded
 
@@ -57,7 +59,8 @@ def is_link(path, use_sudo=False, verbose=False):
 def first(*args, **kwargs):
     """
     Given one or more file paths, returns first one found, or None if none
-    exist. May specify ``use_sudo`` and ``verbose`` which are passed to `exists`.
+    exist. May specify ``use_sudo`` and ``verbose`` which are passed to
+    `exists`.
     """
     for directory in args:
         if exists(directory, **kwargs):
@@ -66,7 +69,7 @@ def first(*args, **kwargs):
 
 def upload_template(filename, destination, context=None, use_jinja=False,
     template_dir=None, use_sudo=False, backup=True, mirror_local_mode=False,
-    mode=None, pty=None):
+    mode=None, pty=None, keep_trailing_newline=False, temp_dir=''):
     """
     Render and upload a template text file to a remote host.
 
@@ -90,18 +93,26 @@ def upload_template(filename, destination, context=None, use_jinja=False,
     By default, the file will be copied to ``destination`` as the logged-in
     user; specify ``use_sudo=True`` to use `sudo` instead.
 
-    The ``mirror_local_mode`` and ``mode`` kwargs are passed directly to an
-    internal `~fabric.operations.put` call; please see its documentation for
-    details on these two options.
+    The ``mirror_local_mode``, ``mode``, and ``temp_dir`` kwargs are passed
+    directly to an internal `~fabric.operations.put` call; please see its
+    documentation for details on these two options.
 
     The ``pty`` kwarg will be passed verbatim to any internal
     `~fabric.operations.run`/`~fabric.operations.sudo` calls, such as those
     used for testing directory-ness, making backups, etc.
 
+    The ``keep_trailing_newline`` kwarg will be passed when creating
+    Jinja2 Environment which is False by default, same as Jinja2's
+    behaviour.
+
     .. versionchanged:: 1.1
         Added the ``backup``, ``mirror_local_mode`` and ``mode`` kwargs.
     .. versionchanged:: 1.9
         Added the ``pty`` kwarg.
+    .. versionchanged:: 1.11
+        Added the ``keep_trailing_newline`` kwarg.
+    .. versionchanged:: 1.11
+        Added the  ``temp_dir`` kwarg.
     """
     func = use_sudo and sudo or run
     if pty is not None:
@@ -115,7 +126,7 @@ def upload_template(filename, destination, context=None, use_jinja=False,
     # Use mode kwarg to implement mirror_local_mode, again due to using
     # StringIO
     if mirror_local_mode and mode is None:
-        mode = os.stat(filename).st_mode
+        mode = os.stat(apply_lcwd(filename, env)).st_mode
         # To prevent put() from trying to do this
         # logic itself
         mirror_local_mode = False
@@ -127,7 +138,8 @@ def upload_template(filename, destination, context=None, use_jinja=False,
             template_dir = template_dir or os.getcwd()
             template_dir = apply_lcwd(template_dir, env)
             from jinja2 import Environment, FileSystemLoader
-            jenv = Environment(loader=FileSystemLoader(template_dir))
+            jenv = Environment(loader=FileSystemLoader(template_dir),
+                               keep_trailing_newline=keep_trailing_newline)
             text = jenv.get_template(filename).render(**context or {})
             # Force to a byte representation of Unicode, or str()ification
             # within Paramiko's SFTP machinery may cause decode issues for
@@ -138,6 +150,8 @@ def upload_template(filename, destination, context=None, use_jinja=False,
             tb = traceback.format_exc()
             abort(tb + "\nUnable to import Jinja2 -- see above.")
     else:
+        if template_dir:
+            filename = os.path.join(template_dir, filename)
         filename = apply_lcwd(filename, env)
         with open(os.path.expanduser(filename)) as inputfile:
             text = inputfile.read()
@@ -148,13 +162,17 @@ def upload_template(filename, destination, context=None, use_jinja=False,
     if backup and exists(destination):
         func("cp %s{,.bak}" % _expand_path(destination))
 
+    if six.PY3 is True and isinstance(text, bytes):
+        text = text.decode('utf-8')
+
     # Upload the file.
     return put(
-        local_path=StringIO(text),
+        local_path=six.StringIO(text),
         remote_path=destination,
         use_sudo=use_sudo,
         mirror_local_mode=mirror_local_mode,
-        mode=mode
+        mode=mode,
+        temp_dir=temp_dir
     )
 
 
@@ -210,7 +228,7 @@ def sed(filename, before, after, limit='', use_sudo=False, backup='.bak',
     # Test the OS because of differences between sed versions
 
     with hide('running', 'stdout'):
-        platform = run("uname")
+        platform = run("uname", shell=False, pty=False)
     if platform in ('NetBSD', 'OpenBSD', 'QNX'):
         # Attempt to protect against failures/collisions
         hasher = hashlib.sha1()
@@ -314,7 +332,7 @@ def comment(filename, regex, use_sudo=False, char='#', backup='.bak',
 
 
 def contains(filename, text, exact=False, use_sudo=False, escape=True,
-    shell=False):
+    shell=False, case_sensitive=True):
     """
     Return True if ``filename`` contains ``text`` (which may be a regex.)
 
@@ -334,7 +352,9 @@ def contains(filename, text, exact=False, use_sudo=False, escape=True,
     added.)
 
     The ``shell`` argument will be eventually passed to ``run/sudo``. See
-    description of the same argumnet in ``~fabric.contrib.sed`` for details.
+    description of the same argument in ``~fabric.contrib.sed`` for details.
+
+    If ``case_sensitive`` is False, the `-i` flag will be passed to ``egrep``.
 
     .. versionchanged:: 1.0
         Swapped the order of the ``filename`` and ``text`` arguments to be
@@ -346,6 +366,8 @@ def contains(filename, text, exact=False, use_sudo=False, escape=True,
         Added ``escape`` keyword argument.
     .. versionadded:: 1.6
         Added the ``shell`` keyword argument.
+    .. versionadded:: 1.11
+        Added the ``case_sensitive`` keyword argument.
     """
     func = use_sudo and sudo or run
     if escape:
@@ -354,6 +376,8 @@ def contains(filename, text, exact=False, use_sudo=False, escape=True,
             text = "^%s$" % text
     with settings(hide('everything'), warn_only=True):
         egrep_cmd = 'egrep "%s" %s' % (text, _expand_path(filename))
+        if not case_sensitive:
+            egrep_cmd = egrep_cmd.replace('egrep', 'egrep -i', 1)
         return func(egrep_cmd, shell=shell).succeeded
 
 
@@ -397,7 +421,7 @@ def append(filename, text, use_sudo=False, partial=False, escape=True,
     """
     func = use_sudo and sudo or run
     # Normalize non-list input to be a list
-    if isinstance(text, basestring):
+    if isinstance(text, six.string_types):
         text = [text]
     for line in text:
         regex = '^' + _escape_for_regex(line)  + ('' if partial else '$')
@@ -410,14 +434,52 @@ def append(filename, text, use_sudo=False, partial=False, escape=True,
 
 def _escape_for_regex(text):
     """Escape ``text`` to allow literal matching using egrep"""
-    regex = re.escape(text)
-    # Seems like double escaping is needed for \
-    regex = regex.replace('\\\\', '\\\\\\')
-    # Triple-escaping seems to be required for $ signs
-    regex = regex.replace(r'\$', r'\\\$')
-    # Whereas single quotes should not be escaped
-    regex = regex.replace(r"\'", "'")
-    return regex
+    re_specials = '\\^$|(){}[]*+?.'
+    sh_specials = '\\$`"'
+    re_chars = []
+    sh_chars = []
+
+    for c in text:
+        if c in re_specials:
+            re_chars.append('\\')
+        re_chars.append(c)
+
+    for c in re_chars:
+        if c in sh_specials:
+            sh_chars.append('\\')
+        sh_chars.append(c)
+
+    return ''.join(sh_chars)
+
+def is_win():
+    """
+    Return True if remote SSH server is running Windows, False otherwise.
+
+    The idea is based on echoing quoted text: \*NIX systems will echo quoted
+    text only, while Windows echoes quotation marks as well.
+    """
+    with settings(hide('everything'), warn_only=True):
+        return '"' in run('echo "Will you echo quotation marks"')
+
+def is_win():
+    """
+    Return True if remote SSH server is running Windows, False otherwise.
+
+    The idea is based on echoing quoted text: \*NIX systems will echo quoted
+    text only, while Windows echoes quotation marks as well.
+    """
+    with settings(hide('everything'), warn_only=True):
+        return '"' in run('echo "Will you echo quotation marks"')
 
 def _expand_path(path):
-    return '"$(echo %s)"' % path
+    """
+    Return a path expansion
+
+    E.g.    ~/some/path     ->  /home/myuser/some/path
+            /user/\*/share   ->  /user/local/share
+    More examples can be found here: http://linuxcommand.org/lc3_lts0080.php
+
+    .. versionchanged:: 1.0
+        Avoid breaking remote Windows commands which does not support expansion.
+    """
+    return path if is_win() else '"$(echo %s)"' % path
