@@ -1,11 +1,8 @@
 """
 Useful non-core functionality, e.g. functions composing multiple operations.
 """
-from __future__ import with_statement
-
 from os import getcwd, sep
 import os.path
-from datetime import datetime
 from tempfile import mkdtemp
 
 from fabric.network import needs_host, key_filenames, normalize
@@ -16,8 +13,17 @@ from fabric.context_managers import cd
 __all__ = ['rsync_project', 'upload_project']
 
 @needs_host
-def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
-    extra_opts='', ssh_opts='', capture=False, upload=True):
+def rsync_project(
+    remote_dir,
+    local_dir=None,
+    exclude=(),
+    delete=False,
+    extra_opts='',
+    ssh_opts='',
+    capture=False,
+    upload=True,
+    default_opts='-pthrvz'
+):
     """
     Synchronize a remote directory with the current project directory via rsync.
 
@@ -35,6 +41,10 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
     output of that function call; thus it will return the stdout, if any, of
     the resultant ``rsync`` call.
 
+    ``rsync_project()`` uses the current Fabric connection parameters (user,
+    host, port) by default, adding them to rsync's ssh options (then mixing in
+    ``ssh_opts``, if given -- see below.)
+
     ``rsync_project()`` takes the following parameters:
 
     * ``remote_dir``: the only required parameter, this is the path to the
@@ -43,7 +53,7 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
 
         * If ``local_dir`` ends with a trailing slash, the files will be
           dropped inside of ``remote_dir``. E.g.
-          ``rsync_project("/home/username/project", "foldername/")`` will drop
+          ``rsync_project("/home/username/project/", "foldername/")`` will drop
           the contents of ``foldername`` inside of ``/home/username/project``.
         * If ``local_dir`` does **not** end with a trailing slash (and this
           includes the default scenario, when ``local_dir`` is not specified),
@@ -70,6 +80,8 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
     * ``capture``: Sent directly into an inner `~fabric.operations.local` call.
     * ``upload``: a boolean controlling whether file synchronization is
       performed up or downstream. Upstream by default.
+    * ``default_opts``: the default rsync options ``-pthrvz``, override if
+      desired (e.g. to remove verbosity, etc).
 
     Furthermore, this function transparently honors Fabric's port and SSH key
     settings. Calling this function when the current host string contains a
@@ -80,12 +92,14 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
     constructed by this function is the following::
 
         rsync [--delete] [--exclude exclude[0][, --exclude[1][, ...]]] \\
-            -pthrvz [extra_opts] <local_dir> <host_string>:<remote_dir>
+            [default_opts] [extra_opts] <local_dir> <host_string>:<remote_dir>
 
     .. versionadded:: 1.4.0
         The ``ssh_opts`` keyword argument.
     .. versionadded:: 1.4.1
         The ``capture`` keyword argument.
+    .. versionadded:: 1.8.0
+        The ``default_opts`` keyword argument.
     """
     # Turn single-string exclude into a one-item list for consistency
     if not hasattr(exclude, '__iter__'):
@@ -104,7 +118,16 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
     port_string = "-p %s" % port
     # RSH
     rsh_string = ""
-    rsh_parts = [key_string, port_string, ssh_opts]
+    if env.gateway is None:
+        gateway_opts = ""
+    else:
+        gw_user, gw_host, gw_port = normalize(env.gateway)
+        gw_str = "-A -o \"ProxyCommand=ssh %s -p %s %s@%s nc %s %s\""
+        gateway_opts = gw_str % (
+            key_string, gw_port, gw_user, gw_host, host, port
+        )
+
+    rsh_parts = [key_string, port_string, ssh_opts, gateway_opts]
     if any(rsh_parts):
         rsh_string = "--rsh='ssh %s'" % " ".join(rsh_parts)
     # Set up options part of string
@@ -112,9 +135,10 @@ def rsync_project(remote_dir, local_dir=None, exclude=(), delete=False,
         'delete': '--delete' if delete else '',
         'exclude': exclude_opts % exclusions,
         'rsh': rsh_string,
-        'extra': extra_opts
+        'default': default_opts,
+        'extra': extra_opts,
     }
-    options = "%(delete)s%(exclude)s -pthrvz %(extra)s %(rsh)s" % options_map
+    options = "%(delete)s%(exclude)s %(default)s %(extra)s %(rsh)s" % options_map
     # Get local directory
     if local_dir is None:
         local_dir = '../' + getcwd().split(sep)[-1]
@@ -146,8 +170,8 @@ def upload_project(local_dir=None, remote_dir="", use_sudo=False):
     a copy of ``local_dir`` will appear as a subdirectory of ``remote_dir``)
     and defaults to the remote user's home directory.
 
-    ``use_sudo`` specifies which method should be used when executing commands 
-    remotely. ``sudo`` will be used if use_sudo is True, otherwise ``run`` will 
+    ``use_sudo`` specifies which method should be used when executing commands
+    remotely. ``sudo`` will be used if use_sudo is True, otherwise ``run`` will
     be used.
 
     This function makes use of the ``tar`` and ``gzip`` programs/libraries,
@@ -169,6 +193,7 @@ def upload_project(local_dir=None, remote_dir="", use_sudo=False):
     local_dir = local_dir.rstrip(os.sep)
 
     local_path, local_name = os.path.split(local_dir)
+    local_path = local_path or '.'
     tar_file = "%s.tar.gz" % local_name
     target_tar = os.path.join(remote_dir, tar_file)
     tmp_folder = mkdtemp()
