@@ -9,12 +9,15 @@ The other callables defined in this module are internal only. Anything useful
 to individuals leveraging Fabric as a library, should be kept elsewhere.
 """
 import getpass
-from operator import isMappingType
+from collections import Mapping
+import inspect
 from optparse import OptionParser
 import os
 import sys
 import types
 
+import six
+
 # For checking callables against the API, & easy mocking
 from fabric import api, state, colors
 from fabric.contrib import console, files, project
@@ -25,11 +28,15 @@ from fabric.tasks import Task, execute, get_task_details
 from fabric.task_utils import _Dict, crawl
 from fabric.utils import abort, indent, warn, _pty_size
 
+try:
+    reduce
+except NameError:
+    from functools import reduce
 
 # One-time calculation of "all internal callables" to avoid doing this on every
 # check of a given fabfile callable (in is_classic_task()).
 _modules = [api, project, files, console, colors]
-_internals = reduce(lambda x, y: x + filter(callable, vars(y).values()),
+_internals = reduce(lambda x, y: x + list(filter(callable, vars(y).values())),
     _modules,
     []
 )
@@ -74,9 +81,10 @@ def _is_package(path):
     """
     Is the given path a Python package?
     """
+    _exists = lambda s: os.path.exists(os.path.join(path, s))
     return (
         os.path.isdir(path)
-        and os.path.exists(os.path.join(path, '__init__.py'))
+        and (_exists('__init__.py') or _exists('__init__.pyc'))
     )
 
 
@@ -125,6 +133,7 @@ def is_classic_task(tup):
             callable(func)
             and (func not in _internals)
             and not name.startswith('_')
+            and not (inspect.isclass(func) and issubclass(func, Exception))
         )
     # Handle poorly behaved __eq__ implementations
     except (ValueError, TypeError):
@@ -189,7 +198,7 @@ def load_tasks_from_module(imported):
         imported_vars = [(name, imported_vars[name]) for name in \
                          imported_vars if name in imported_vars["__all__"]]
     else:
-        imported_vars = imported_vars.items()
+        imported_vars = list(imported_vars.items())
     # Return a two-tuple value.  First is the documentation, second is a
     # dictionary of callables only (and don't include Fab operations or
     # underscored callables)
@@ -300,6 +309,12 @@ def parse_options():
         help="Force password prompt up-front"
     )
 
+    parser.add_option('--initial-sudo-password-prompt',
+        action='store_true',
+        default=False,
+        help="Force sudo password prompt up-front"
+    )
+
     # List Fab commands found in loaded fabfiles/source files
     parser.add_option('-l', '--list',
         action='store_true',
@@ -358,10 +373,10 @@ def _is_task(name, value):
 
 def _sift_tasks(mapping):
     tasks, collections = [], []
-    for name, value in mapping.iteritems():
+    for name, value in six.iteritems(mapping):
         if _is_task(name, value):
             tasks.append(name)
-        elif isMappingType(value):
+        elif isinstance(value, Mapping):
             collections.append(name)
     tasks = sorted(tasks)
     collections = sorted(collections)
@@ -389,7 +404,7 @@ def _print_docstring(docstrings, name):
     if not docstrings:
         return False
     docstring = crawl(name, state.commands).__doc__
-    if isinstance(docstring, basestring):
+    if isinstance(docstring, six.string_types):
         return docstring
 
 
@@ -406,7 +421,7 @@ def _normal_list(docstrings=True):
         docstring = _print_docstring(docstrings, name)
         if docstring:
             lines = filter(None, docstring.splitlines())
-            first_line = lines[0].strip()
+            first_line = list(lines)[0].strip()
             # Truncate it if it's longer than N chars
             size = max_width - (max_len + len(sep) + len(trail))
             if len(first_line) > size:
@@ -622,7 +637,7 @@ def main(fabfile_locations=None):
         # Handle --hosts, --roles, --exclude-hosts (comma separated string =>
         # list)
         for key in ['hosts', 'roles', 'exclude_hosts']:
-            if key in state.env and isinstance(state.env[key], basestring):
+            if key in state.env and isinstance(state.env[key], six.string_types):
                 state.env[key] = state.env[key].split(',')
 
         # Feed the env.tasks : tasks that are asked to be executed.
@@ -633,7 +648,7 @@ def main(fabfile_locations=None):
 
         # Handle version number option
         if options.show_version:
-            print("Fabric %s" % state.env.version)
+            print("Fabric39 %s" % state.env.version)
             print("Paramiko %s" % ssh.__version__)
             sys.exit(0)
 
@@ -709,7 +724,7 @@ Remember that -f can be used to specify fabfile path, and use -h for help.""")
                 unknown_commands.append(tup[0])
 
         # Abort if any unknown commands were specified
-        if unknown_commands:
+        if unknown_commands and not state.env.get('skip_unknown_tasks', False):
             warn("Command(s) not found:\n%s" \
                 % indent(unknown_commands))
             show_commands(None, options.list_format, 1)
@@ -729,6 +744,11 @@ Remember that -f can be used to specify fabfile path, and use -h for help.""")
             prompt = "Initial value for env.password: "
             state.env.password = getpass.getpass(prompt)
 
+        # Ditto sudo_password
+        if options.initial_sudo_password_prompt:
+            prompt = "Initial value for env.sudo_password: "
+            state.env.sudo_password = getpass.getpass(prompt)
+
         if state.output.debug:
             names = ", ".join(x[0] for x in commands_to_run)
             print("Commands to run: %s" % names)
