@@ -2,27 +2,24 @@
 Classes and subroutines dealing with network connections and related topics.
 """
 
-from __future__ import with_statement
-
 from functools import wraps
 import getpass
 import os
 import re
 import time
+import six
 import socket
 import sys
-from StringIO import StringIO
-
 
 from fabric.auth import get_password, set_password
-from fabric.utils import abort, handle_prompt_abort, warn
+from fabric.utils import handle_prompt_abort, warn
 from fabric.exceptions import NetworkError
 
 try:
     import warnings
     warnings.simplefilter('ignore', DeprecationWarning)
     import paramiko as ssh
-except ImportError, e:
+except ImportError:
     import traceback
     traceback.print_exc()
     msg = """
@@ -34,7 +31,7 @@ Please make sure all dependencies are installed and importable.
 
 
 ipv6_regex = re.compile(
-    '^\[?(?P<host>[0-9A-Fa-f:]+(?:%[a-z]+\d+)?)\]?(:(?P<port>\d+))?$')
+    r'^\[?(?P<host>[0-9A-Fa-f:]+(?:%[a-z]+\d+)?)\]?(:(?P<port>\d+))?$')
 
 
 def direct_tcpip(client, host, port):
@@ -89,7 +86,7 @@ def get_gateway(host, port, cache, replace=False):
         # ensure initial gateway connection
         if replace or gateway not in cache:
             if output.debug:
-                print "Creating new gateway connection to %r" % gateway
+                print("Creating new gateway connection to %r" % gateway)
             cache[gateway] = connect(*normalize(gateway) + (cache, False))
         # now we should have an open gw connection and can ask it for a
         # direct-tcpip channel to the real target. (bypass cache's own
@@ -214,16 +211,16 @@ def key_filenames():
     from fabric.state import env
     keys = env.key_filename
     # For ease of use, coerce stringish key filename into list
-    if isinstance(env.key_filename, basestring) or env.key_filename is None:
+    if isinstance(env.key_filename, six.string_types) or env.key_filename is None:
         keys = [keys]
     # Strip out any empty strings (such as the default value...meh)
-    keys = filter(bool, keys)
+    keys = list(filter(bool, keys))
     # Honor SSH config
     conf = ssh_config()
     if 'identityfile' in conf:
         # Assume a list here as we require Paramiko 1.10+
         keys.extend(conf['identityfile'])
-    return map(os.path.expanduser, keys)
+    return list(map(os.path.expanduser, keys))
 
 
 def key_from_env(passphrase=None):
@@ -242,11 +239,11 @@ def key_from_env(passphrase=None):
             if output.debug:
                 sys.stderr.write("Trying to load it as %s\n" % pkey_class)
             try:
-                return pkey_class.from_private_key(StringIO(env.key), passphrase)
-            except Exception, e:
+                return pkey_class.from_private_key(six.StringIO(env.key), passphrase)
+            except Exception as e:
                 # File is valid key, but is encrypted: raise it, this will
                 # cause cxn loop to prompt for passphrase & retry
-                if 'Private key file is encrypted' in e:
+                if 'Private key file is encrypted' in str(e):
                     raise
                 # Otherwise, it probably means it wasn't a valid key of this
                 # type, so try the next one.
@@ -472,7 +469,7 @@ def connect(user, host, port, cache, seek_gateway=True):
                 sock=sock,
             )
             for suffix in ('auth', 'deleg_creds', 'kex'):
-                name = 'gss_{0}'.format(suffix)
+                name = "gss_" + suffix
                 val = env.get(name, None)
                 if val is not None:
                     kwargs[name] = val
@@ -489,20 +486,24 @@ def connect(user, host, port, cache, seek_gateway=True):
         # BadHostKeyException corresponds to key mismatch, i.e. what on the
         # command line results in the big banner error about man-in-the-middle
         # attacks.
-        except ssh.BadHostKeyException, e:
+        except ssh.BadHostKeyException as e:
             raise NetworkError("Host key for %s did not match pre-existing key! Server's key was changed recently, or possible man-in-the-middle attack." % host, e)
         # Prompt for new password to try on auth failure
         except (
             ssh.AuthenticationException,
             ssh.PasswordRequiredException,
             ssh.SSHException
-        ), e:
+        ) as e:
             msg = str(e)
             # If we get SSHExceptionError and the exception message indicates
             # SSH protocol banner read failures, assume it's caused by the
             # server load and try again.
-            if e.__class__ is ssh.SSHException \
-                and msg == 'Error reading SSH protocol banner':
+            #
+            # If we are using a gateway, we will get a ChannelException if
+            # connection to the downstream host fails. We should retry.
+            if (e.__class__ is ssh.SSHException \
+                and msg == 'Error reading SSH protocol banner') \
+                or e.__class__ is ssh.ChannelException:
                 if _tried_enough(tries):
                     raise NetworkError(msg, e)
                 continue
@@ -520,9 +521,15 @@ def connect(user, host, port, cache, seek_gateway=True):
             #
             # This also holds true for rejected/unknown host keys: we have to
             # guess based on other heuristics.
-            if e.__class__ is ssh.SSHException \
-                and (password or msg.startswith('Unknown server')) \
-                and not is_key_load_error(e):
+            if (
+                e.__class__ is ssh.SSHException
+                and (
+                    password
+                    or msg.startswith('Unknown server')
+                    or "not found in known_hosts" in msg
+                )
+                and not is_key_load_error(e)
+            ):
                 raise NetworkError(msg, e)
 
             # Otherwise, assume an auth exception, and prompt for new/better
@@ -567,11 +574,11 @@ def connect(user, host, port, cache, seek_gateway=True):
             print('')
             sys.exit(0)
         # Handle DNS error / name lookup failure
-        except socket.gaierror, e:
+        except socket.gaierror as e:
             raise NetworkError('Name lookup failed for %s' % host, e)
         # Handle timeouts and retries, including generic errors
         # NOTE: In 2.6, socket.error subclasses IOError
-        except socket.error, e:
+        except socket.error as e:
             not_timeout = type(e) is not socket.timeout
             giving_up = _tried_enough(tries)
             # Baseline error msg for when debug is off
@@ -594,7 +601,7 @@ def connect(user, host, port, cache, seek_gateway=True):
             # Override eror msg if we were retrying other errors
             if not_timeout:
                 msg = "Low level socket error connecting to host %s on port %s: %s" % (
-                    host, port, e[1]
+                    host, port, e.args[1]
                 )
             # Here, all attempts failed. Tweak error msg to show # tries.
             # TODO: find good humanization module, jeez
@@ -611,7 +618,9 @@ def connect(user, host, port, cache, seek_gateway=True):
 def _password_prompt(prompt, stream):
     # NOTE: Using encode-to-ascii to prevent (Windows, at least) getpass from
     # choking if given Unicode.
-    return getpass.getpass(prompt.encode('ascii', 'ignore'), stream)
+    if six.PY3 is False:
+        prompt = prompt.encode('ascii', 'ignore')
+    return getpass.getpass(prompt, stream)
 
 def prompt_for_password(prompt=None, no_colon=False, stream=None):
     """
@@ -671,8 +680,14 @@ def needs_host(func):
     def host_prompting_wrapper(*args, **kwargs):
         while not env.get('host_string', False):
             handle_prompt_abort("the target host connection string")
-            host_string = raw_input("No hosts found. Please specify (single)"
-                                    " host string for connection: ")
+            prompt = "No hosts found. Please specify (single) " \
+                "host string for connection: "
+            # WARNING: do not use six.moves.input, because test cases to not
+            # overwrite that method with a faked method from Fudge
+            if six.PY3 is True:
+                host_string = input(prompt)
+            else:
+                host_string = raw_input(prompt)
             env.update(to_dict(host_string))
         return func(*args, **kwargs)
     host_prompting_wrapper.undecorated = func
@@ -688,7 +703,7 @@ def disconnect_all():
     """
     from fabric.state import connections, output
     # Explicitly disconnect from all servers
-    for key in connections.keys():
+    for key in list(connections.keys()):
         if output.status:
             # Here we can't use the py3k print(x, end=" ")
             # because 2.5 backwards compatibility
