@@ -1,10 +1,11 @@
 from __future__ import with_statement
 
-import hashlib
 import os
 import posixpath
+import six
 import stat
 import re
+import uuid
 from fnmatch import filter as fnfilter
 
 from fabric.state import output, connections, env
@@ -12,6 +13,9 @@ from fabric.utils import warn
 from fabric.context_managers import settings
 
 
+# TODO: use self.sftp.listdir_iter on Paramiko 1.15+
+
+
 def _format_local(local_path, local_is_path):
     """Format a path for log output"""
     if local_is_path:
@@ -59,7 +63,7 @@ class SFTP(object):
         rlist = self.ftp.listdir(dirpart)
 
         names = fnfilter([f for f in rlist if not f[0] == '.'], pattern)
-        ret = [path]
+        ret = []
         if len(names):
             s = '/'
             ret = [dirpart.rstrip(s) + s + name.lstrip(s) for name in names]
@@ -79,7 +83,7 @@ class SFTP(object):
             # Note that listdir and error are globals in this module due to
             # earlier import-*.
             names = self.ftp.listdir(top)
-        except Exception, err:
+        except Exception as err:
             if onerror is not None:
                 onerror(err)
             return
@@ -127,9 +131,10 @@ class SFTP(object):
         }
 
         if local_is_path:
-            # Naive fix to issue #711
-            escaped_path = re.sub(r'(%[^()]*\w)', r'%\1', local_path)
-            local_path = os.path.abspath(escaped_path % path_vars )
+            # Fix for issue #711 and #1348 - escape %'s as well as possible.
+            format_re = r'(%%(?!\((?:%s)\)\w))' % '|'.join(path_vars.keys())
+            escaped_path = re.sub(format_re, r'%\1', local_path)
+            local_path = os.path.abspath(escaped_path % path_vars)
 
             # Ensure we give ssh.SFTPCLient a file by prepending and/or
             # creating local directories as appropriate.
@@ -154,11 +159,7 @@ class SFTP(object):
         # path in the default remote CWD (which, typically, the login user will
         # have write permissions on) in order to sudo(cp) it.
         if use_sudo:
-            target_path = remote_path
-            hasher = hashlib.sha1()
-            hasher.update(env.host_string)
-            hasher.update(target_path)
-            target_path = posixpath.join(temp_dir, hasher.hexdigest())
+            target_path = posixpath.join(temp_dir, uuid.uuid4().hex)
             # Temporarily nuke 'cwd' so sudo() doesn't "cd" its mv command.
             # (The target path has already been cwd-ified elsewhere.)
             with settings(hide('everything'), cwd=""):
@@ -166,7 +167,7 @@ class SFTP(object):
                 # The user should always own the copied file.
                 sudo('chown %s "%s"' % (env.user, target_path))
                 # Only root and the user has the right to read the file
-                sudo('chmod %o "%s"' % (0400, target_path))
+                sudo('chmod 400 "%s"' % target_path)
                 remote_path = target_path
 
         try:
@@ -246,10 +247,7 @@ class SFTP(object):
         # have write permissions on) in order to sudo(mv) it later.
         if use_sudo:
             target_path = remote_path
-            hasher = hashlib.sha1()
-            hasher.update(env.host_string)
-            hasher.update(target_path)
-            remote_path = posixpath.join(temp_dir, hasher.hexdigest())
+            remote_path = posixpath.join(temp_dir, uuid.uuid4().hex)
         # Read, ensuring we handle file-like objects correct re: seek pointer
         putter = self.ftp.put
         if not local_is_path:
@@ -263,13 +261,13 @@ class SFTP(object):
         if (local_is_path and mirror_local_mode) or (mode is not None):
             lmode = os.stat(local_path).st_mode if mirror_local_mode else mode
             # Cast to octal integer in case of string
-            if isinstance(lmode, basestring):
+            if isinstance(lmode, six.string_types):
                 lmode = int(lmode, 8)
-            lmode = lmode & 07777
+            lmode = lmode & int('0o7777', 8)
             rmode = rattrs.st_mode
             # Only bitshift if we actually got an rmode
             if rmode is not None:
-                rmode = (rmode & 07777)
+                rmode = rmode & int('0o7777', 8)
             if lmode != rmode:
                 if use_sudo:
                     # Temporarily nuke 'cwd' so sudo() doesn't "cd" its mv
