@@ -1,13 +1,13 @@
 from __future__ import with_statement
 
-from functools import wraps
 import inspect
+import six
 import sys
 import textwrap
 
 from fabric import state
 from fabric.utils import abort, warn, error
-from fabric.network import to_dict, normalize_to_string, disconnect_all
+from fabric.network import to_dict, disconnect_all
 from fabric.context_managers import settings
 from fabric.job_queue import JobQueue
 from fabric.task_utils import crawl, merge, parse_kwargs
@@ -122,7 +122,7 @@ class Task(object):
         # from the CLI or from module-level code). This will be the empty list
         # if these have not been set -- which is fine, this method should
         # return an empty list if no hosts have been set anywhere.
-        env_vars = map(_get_list(env), "hosts roles exclude_hosts".split())
+        env_vars = list(map(_get_list(env), "hosts roles exclude_hosts".split()))
         env_vars.append(roledefs)
         return merge(*env_vars), env.get('roles', [])
 
@@ -177,7 +177,10 @@ class WrappedCallableTask(Task):
         return getattr(self.wrapped, k)
 
     def __details__(self):
-        return get_task_details(self.wrapped)
+        orig = self
+        while 'wrapped' in orig.__dict__:
+            orig = orig.__dict__.get('wrapped')
+        return get_task_details(orig)
 
 
 def requires_parallel(task):
@@ -206,7 +209,6 @@ def _parallel_tasks(commands_to_run):
 def _is_network_error_ignored():
     return not state.env.use_exceptions_for['network'] and state.env.skip_bad_hosts
 
-
 def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
     """
     Primary single-host work body of execute()
@@ -230,14 +232,15 @@ def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
         # * nukes the connection cache to prevent shared-access problems
         # * knows how to send the tasks' return value back over a Queue
         # * captures exceptions raised by the task
-        def inner(args, kwargs, queue, name, env):
+
+        def _inner(args, kwargs, queue, name, env):
             state.env.update(env)
             def submit(result):
                 queue.put({'name': name, 'result': result})
             try:
                 state.connections.clear()
                 submit(task.run(*args, **kwargs))
-            except BaseException, e: # We really do want to capture everything
+            except BaseException as e:  # We really do want to capture everything
                 # SystemExit implies use of abort(), which prints its own
                 # traceback, host info etc -- so we don't want to double up
                 # on that. For everything else, though, we need to make
@@ -263,7 +266,7 @@ def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
             'name': name,
             'env': local_env,
         }
-        p = multiprocessing.Process(target=inner, kwargs=kwarg_dict)
+        p = multiprocessing.Process(target=_inner, kwargs=kwarg_dict)
         # Name/id is host string
         p.name = name
         # Add to queue
@@ -330,7 +333,12 @@ def execute(task, *args, **kwargs):
         my_env['command'] = task
         task = crawl(task, state.commands)
         if task is None:
-            abort("%r is not callable or a valid task name" % (task,))
+            msg = "%r is not callable or a valid task name" % (my_env['command'],)
+            if state.env.get('skip_unknown_tasks', False):
+                warn(msg)
+                return
+            else:
+                abort(msg)
     # Set env.command if we were given a real function or callable task obj
     else:
         dunder_name = getattr(task, '__name__', None)
@@ -349,13 +357,13 @@ def execute(task, *args, **kwargs):
         # Import multiprocessing if needed, erroring out usefully
         # if it can't.
         try:
-            import multiprocessing
+            from pathos.helpers import mp as multiprocessing
         except ImportError:
             import traceback
             tb = traceback.format_exc()
             abort(tb + """
     At least one task needs to be run in parallel, but the
-    multiprocessing module cannot be imported (see above
+    pathos.helpers.mp module cannot be imported (see above
     traceback.) Please make sure the module is installed
     or that the above ImportError is fixed.""")
     else:
@@ -378,7 +386,7 @@ def execute(task, *args, **kwargs):
                     task, host, my_env, args, new_kwargs, jobs, queue,
                     multiprocessing
                 )
-            except NetworkError, e:
+            except NetworkError as e:
                 results[host] = e
                 # Backwards compat test re: whether to use an exception or
                 # abort
@@ -402,8 +410,8 @@ def execute(task, *args, **kwargs):
             # This prevents Fabric from continuing on to any other tasks.
             # Otherwise, pull in results from the child run.
             ran_jobs = jobs.run()
-            for name, d in ran_jobs.iteritems():
-                if d['exit_code'] != 0:
+            for name, d in six.iteritems(ran_jobs):
+                if d['exit_code'] not in (0, None):
                     if isinstance(d['results'], NetworkError) and \
                             _is_network_error_ignored():
                         error(d['results'].message, func=warn, exception=d['results'].wrapped)
@@ -419,4 +427,4 @@ def execute(task, *args, **kwargs):
             results['<local-only>'] = task.run(*args, **new_kwargs)
     # Return what we can from the inner task executions
 
-    return results
+    return results
\ No newline at end of file
