@@ -9,7 +9,7 @@ from subprocess import Popen, PIPE
 from os.path import abspath, dirname
 
 
-VERSION = (1, 13, 2, 'final', 0)
+VERSION = (1, 15, 1, 'post', 1)
 
 
 def git_sha():
@@ -51,6 +51,7 @@ def get_version(form='short'):
     tertiary = VERSION[2]
     type_ = VERSION[3]
     final = (type_ == "final")
+    post = (type_ == "post")
     type_num = VERSION[4]
     firsts = "".join([x[0] for x in type_.split()])
 
@@ -61,7 +62,9 @@ def get_version(form='short'):
     v = branch
     if (tertiary or final):
         v += "." + str(tertiary)
-    if not final:
+    if post:
+        v += '.%s%s' % (type_, type_num)
+    elif not final:
         v += firsts
         if type_num:
             v += str(type_num)
