@@ -1,25 +1,19 @@
 from __future__ import with_statement
 
-import copy
-import itertools
 import os
 import re
+import six
 import socket
-import stat
-import sys
 import threading
 import time
-import types
-from StringIO import StringIO
 from functools import wraps
 from Python26SocketServer import BaseRequestHandler, ThreadingMixIn, TCPServer
 
-import paramiko as ssh
-
 from fabric.operations import _sudo_prefix
-from fabric.api import env, hide
+from fabric.context_managers import hide
+from fabric.state import env
 from fabric.thread_handling import ThreadHandler
-from fabric.network import disconnect_all
+from fabric.network import disconnect_all, ssh
 
 from fake_filesystem import FakeFilesystem, FakeFile
 
@@ -60,7 +54,7 @@ tests""",
     "both_streams": [
         "stdout",
         "stderr"
-    ]
+    ],
 }
 FILES = FakeFilesystem({
     '/file.txt': 'contents',
@@ -92,7 +86,7 @@ def _equalize(lists, fillval=None):
     """
     Pad all given list items in ``lists`` to be the same length.
     """
-    lists = map(list, lists)
+    lists = list(map(list, lists))
     upper = max(len(x) for x in lists)
     for lst in lists:
         diff = upper - len(lst)
@@ -101,9 +95,9 @@ def _equalize(lists, fillval=None):
     return lists
 
 
-class ParamikoServer(ssh.ServerInterface):
+class TestServer(ssh.ServerInterface):
     """
-    Test-ready server implementing Paramiko's server interface parent class.
+    Test server implementing the 'ssh' lib's server interface parent class.
 
     Mostly just handles the bare minimum necessary to handle SSH-level things
     such as honoring authentication types and exec/shell/etc requests.
@@ -155,7 +149,7 @@ class SSHServer(ThreadingMixIn, TCPServer):
     """
     def _socket_info(self, addr_tup):
         """
-        Clone of the very top of Paramiko (1.7.6) SSHClient.connect().
+        Clone of the very top of Paramiko 1.7.6 SSHClient.connect().
 
         We must use this in order to make sure that our address family matches
         up with the client side (which we cannot control, and which varies
@@ -395,21 +389,26 @@ def serve_responses(responses, files, passwords, home, pubkeys, port):
             transport.add_server_key(ssh.RSAKey(filename=SERVER_PRIVKEY))
             transport.set_subsystem_handler('sftp', ssh.SFTPServer,
                 sftp_si=FakeSFTPServer)
-            server = ParamikoServer(passwords, home, pubkeys, files)
+            server = TestServer(passwords, home, pubkeys, files)
             transport.start_server(server=server)
             self.ssh_server = server
             self.transport = transport
 
         def split_sudo_prompt(self):
-            prefix = re.escape(_sudo_prefix(None).rstrip()) + ' +'
-            result = re.findall(r'^(%s)?(.*)$' % prefix, self.command)[0]
+            prefix = re.escape(_sudo_prefix(None, None).rstrip()) + ' +'
+            command = self.command
+            if six.PY3 and isinstance(command, bytes):
+                command = command.decode('utf-8')
+
+            result = re.findall(r'^(%s)?(.*)$' % prefix, command)[0]
             self.sudo_prompt, self.command = result
 
         def response(self):
             result = responses[self.command]
             stderr = ""
             status = 0
-            if isinstance(result, types.StringTypes):
+            sleep = 0
+            if isinstance(result, six.string_types):
                 stdout = result
             else:
                 size = len(result)
@@ -419,7 +418,10 @@ def serve_responses(responses, files, passwords, home, pubkeys, port):
                     stdout, stderr = result
                 elif size == 3:
                     stdout, stderr, status = result
+                elif size == 4:
+                    stdout, stderr, status, sleep = result
             stdout, stderr = _equalize((stdout, stderr))
+            time.sleep(sleep)
             return stdout, stderr, status
 
         def sudo_password(self):
@@ -432,6 +434,8 @@ def serve_responses(responses, files, passwords, home, pubkeys, port):
                 # newline
                 self.channel.send('\n')
                 # Test password
+                if six.PY3 is True:
+                    password = password.decode('utf-8')
                 if password == passwords[self.ssh_server.username]:
                     passed = True
                     break
@@ -487,6 +491,6 @@ def server(
                 # Handle subthread exceptions
                 e = worker.exception
                 if e:
-                    raise e[0], e[1], e[2]
+                    six.reraise(e[0], e[1], e[2])
         return inner
     return run_server
