@@ -1,8 +1,7 @@
-from __future__ import with_statement
-
 import copy
+from collections import Mapping
 from functools import partial
-from operator import isMappingType
+import six
 import os.path
 import sys
 
@@ -619,8 +618,8 @@ def name_to_task(name):
 
 def strings_to_tasks(d):
     ret = {}
-    for key, value in d.iteritems():
-        if isMappingType(value):
+    for key, value in six.iteritems(d):
+        if isinstance(value, Mapping):
             val = strings_to_tasks(value)
         else:
             val = name_to_task(value)
