@@ -1,23 +1,29 @@
 from __future__ import with_statement
 
-from contextlib import nested
-from StringIO import StringIO
+import os
+import re
+import shutil
+import six
+import sys
 
-from nose.tools import ok_
-from fudge import with_fakes, Fake
+from nose.tools import ok_, raises
+from fudge import patched_context, with_fakes, Fake
 from fudge.inspector import arg as fudge_arg
+from mock_streams import mock_streams
 from paramiko.sftp_client import SFTPClient  # for patching
 
-from fabric.state import env, output
+from fabric.state import env
+from fabric.context_managers import nested, settings
 from fabric.operations import require, prompt, _sudo_prefix, _shell_wrap, \
     _shell_escape
 from fabric.api import get, put, hide, show, cd, lcd, local, run, sudo, quiet
 from fabric.exceptions import CommandTimeout
+from fabric.sftp import SFTP
 
 from fabric.decorators import with_settings
-from utils import *
-from server import (server, PORT, RESPONSES, FILES, PASSWORDS, CLIENT_PRIVKEY,
-    USER, CLIENT_PRIVKEY_PASSPHRASE)
+from utils import (eq_, aborts, assert_contains, eq_contents,
+                   with_patched_input, FabricTest)
+from server import server, FILES
 
 #
 # require()
@@ -309,8 +315,8 @@ class TestCombineStderr(FabricTest):
         """
         combine_stderr: no kwarg => uses global value (True)
         """
-        output.everything = False
-        r = run("both_streams")
+        with show('everything'):
+            r = run("both_streams")
         # Note: the exact way the streams are jumbled here is an implementation
         # detail of our fake SSH server and may change in the future.
         eq_("ssttddoeurtr", r.stdout)
@@ -321,9 +327,9 @@ class TestCombineStderr(FabricTest):
         """
         combine_stderr: no kwarg => uses global value (False)
         """
-        output.everything = False
-        env.combine_stderr = False
-        r = run("both_streams")
+        with show('everything'):
+            env.combine_stderr = False
+            r = run("both_streams")
         eq_("stdout", r.stdout)
         eq_("stderr", r.stderr)
 
@@ -332,9 +338,9 @@ class TestCombineStderr(FabricTest):
         """
         combine_stderr: True kwarg => overrides global False value
         """
-        output.everything = False
-        env.combine_stderr = False
-        r = run("both_streams", combine_stderr=True)
+        with show('everything'):
+            env.combine_stderr = False
+            r = run("both_streams", combine_stderr=True)
         eq_("ssttddoeurtr", r.stdout)
         eq_(r.stderr, "")
 
@@ -343,9 +349,9 @@ class TestCombineStderr(FabricTest):
         """
         combine_stderr: False kwarg => overrides global True value
         """
-        output.everything = False
-        env.combine_stderr = True
-        r = run("both_streams", combine_stderr=False)
+        with show('everything'):
+            env.combine_stderr = True
+            r = run("both_streams", combine_stderr=False)
         eq_("stdout", r.stdout)
         eq_("stderr", r.stderr)
 
@@ -447,7 +453,7 @@ class TestFileTransfers(FabricTest):
         local = self.path(remote)
         with hide('everything'):
             get(remote, local)
-        eq_contents(local, FILES[remote])
+        eq_contents(local, str(FILES[remote]))
 
     @server(files={'/base/dir with spaces/file': 'stuff!'})
     def test_get_file_from_relative_path_with_spaces(self):
@@ -468,7 +474,7 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             get('file*.txt', self.tmpdir)
         for remote in remotes:
-            eq_contents(self.path(remote), FILES[remote])
+            eq_contents(self.path(remote), str(FILES[remote]))
 
     @server()
     def test_get_single_file_in_folder(self):
@@ -478,7 +484,7 @@ class TestFileTransfers(FabricTest):
         remote = 'folder/file3.txt'
         with hide('everything'):
             get('folder', self.tmpdir)
-        eq_contents(self.path(remote), FILES[remote])
+        eq_contents(self.path(remote), str(FILES[remote]))
 
     @server()
     def test_get_tree(self):
@@ -489,7 +495,7 @@ class TestFileTransfers(FabricTest):
             get('tree', self.tmpdir)
         leaves = filter(lambda x: x[0].startswith('/tree'), FILES.items())
         for path, contents in leaves:
-            eq_contents(self.path(path[1:]), contents)
+            eq_contents(self.path(path[1:]), str(contents))
 
     @server()
     def test_get_tree_with_implicit_local_path(self):
@@ -503,7 +509,7 @@ class TestFileTransfers(FabricTest):
             leaves = filter(lambda x: x[0].startswith('/tree'), FILES.items())
             for path, contents in leaves:
                 path = os.path.join(dirname, path[1:])
-                eq_contents(path, contents)
+                eq_contents(path, str(contents))
                 os.remove(path)
         # Cleanup
         finally:
@@ -537,7 +543,7 @@ class TestFileTransfers(FabricTest):
     @mock_streams('stderr')
     def _invalid_file_obj_situations(self, remote_path):
         with settings(hide('running'), warn_only=True):
-            get(remote_path, StringIO())
+            get(remote_path, six.StringIO())
         assert_contains('is a glob or directory', sys.stderr.getvalue())
 
     def test_glob_and_file_object_invalid(self):
@@ -552,6 +558,23 @@ class TestFileTransfers(FabricTest):
         """
         self._invalid_file_obj_situations('/tree')
 
+    @server()
+    def test_nonexistent_glob_should_not_create_empty_files(self):
+        path = self.path()
+        with settings(hide('everything'), warn_only=True):
+            get('/nope*.txt', path)
+        assert not self.exists_locally(os.path.join(path, 'nope*.txt'))
+
+    @server()
+    def test_nonexistent_glob_raises_error(self):
+        try:
+            with hide('everything', 'aborts'):
+                get('/nope*.txt', self.path())
+        except SystemExit as e:
+            assert 'No such file' in e.message
+        else:
+            assert False
+
     @server()
     def test_get_single_file_absolutely(self):
         """
@@ -560,7 +583,7 @@ class TestFileTransfers(FabricTest):
         target = '/etc/apache2/apache2.conf'
         with hide('everything'):
             get(target, self.tmpdir)
-        eq_contents(self.path(os.path.basename(target)), FILES[target])
+        eq_contents(self.path(os.path.basename(target)), str(FILES[target]))
 
     @server()
     def test_get_file_with_nonexistent_target(self):
@@ -571,7 +594,7 @@ class TestFileTransfers(FabricTest):
         target = 'file.txt'
         with hide('everything'):
             get(target, local)
-        eq_contents(local, FILES[target])
+        eq_contents(local, str(FILES[target]))
 
     @server()
     @mock_streams('stderr')
@@ -586,7 +609,7 @@ class TestFileTransfers(FabricTest):
         with hide('stdout', 'running'):
             get(target, local)
         assert "%s already exists" % local in sys.stderr.getvalue()
-        eq_contents(local, FILES[target])
+        eq_contents(local, str(FILES[target]))
 
     @server()
     def test_get_file_to_directory(self):
@@ -599,7 +622,7 @@ class TestFileTransfers(FabricTest):
         target = 'file.txt'
         with hide('everything'):
             get(target, self.tmpdir)
-        eq_contents(self.path(target), FILES[target])
+        eq_contents(self.path(target), str(FILES[target]))
 
     @server(port=2200)
     @server(port=2201)
@@ -670,11 +693,11 @@ class TestFileTransfers(FabricTest):
         """
         get()'s local_path arg should take file-like objects too
         """
-        fake_file = StringIO()
+        fake_file = six.BytesIO()
         target = '/file.txt'
         with hide('everything'):
             get(target, fake_file)
-        eq_(fake_file.getvalue(), FILES[target])
+        eq_(fake_file.getvalue().decode('utf-8'), str(FILES[target]))
 
     @server()
     def test_get_interpolation_without_host(self):
@@ -701,7 +724,8 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             retval = get('tree', d)
         files = ['file1.txt', 'file2.txt', 'subfolder/file3.txt']
-        eq_(map(lambda x: os.path.join(d, 'tree', x), files), retval)
+        got = sorted(map(lambda x: os.path.join(d, 'tree', x), files))
+        eq_(list(got), sorted(retval))
 
     @server()
     def test_get_returns_none_for_stringio(self):
@@ -709,7 +733,7 @@ class TestFileTransfers(FabricTest):
         get() should return None if local_path is a StringIO
         """
         with hide('everything'):
-            eq_([], get('/file.txt', StringIO()))
+            eq_([], get('/file.txt', six.BytesIO()))
 
     @server()
     def test_get_return_value_failed_attribute(self):
@@ -738,19 +762,16 @@ class TestFileTransfers(FabricTest):
         """
         get(use_sudo=True) works by copying to a temporary path, downloading it and then removing it at the end
         """
-        # the sha1 hash is the unique filename of the file being downloaded. sha1(<filename>)
-        name = "229a29e5693876645e39de0cb0532e43ad73311a"
         fake_run = Fake('_run_command', callable=True, expect_call=True).with_matching_args(
-            'cp -p "/etc/apache2/apache2.conf" "%s"' % name, True, True, None,
+            fudge_arg.startswith('cp -p "/etc/apache2/apache2.conf" "'), True, True, None
         ).next_call().with_matching_args(
-            'chown username "%s"' % name, True, True, None,
+            fudge_arg.startswith('chown username "'), True, True, None,
         ).next_call().with_matching_args(
-            'chmod 400 "%s"' % name, True, True, None,
+            fudge_arg.startswith('chmod 400 "'), True, True, None,
         ).next_call().with_matching_args(
-            'rm -f "%s"' % name, True, True, None,
+            fudge_arg.startswith('rm -f "'), True, True, None,
         )
-        fake_get = Fake('get', callable=True, expect_call=True).with_args(
-            name, fudge_arg.any_value())
+        fake_get = Fake('get', callable=True, expect_call=True)
 
         with hide('everything'):
             with patched_context('fabric.operations', '_run_command', fake_run):
@@ -763,21 +784,19 @@ class TestFileTransfers(FabricTest):
     @with_fakes
     def test_get_use_sudo_temp_dir(self):
         """
-        get(use_sudo=True, temp_dir="/tmp") works by copying to a /tmp/sha1_hash, downloading it and then removing it at the end
+        get(use_sudo=True, temp_dir="/tmp") works by copying to /tmp/..., downloading it and then removing it at the end
         """
-        # the sha1 hash is the unique filename of the file being downloaded. sha1(<filename>)
-        name = "229a29e5693876645e39de0cb0532e43ad73311a"
         fake_run = Fake('_run_command', callable=True, expect_call=True).with_matching_args(
-            'cp -p "/etc/apache2/apache2.conf" "/tmp/%s"' % name, True, True, None,
+            fudge_arg.startswith('cp -p "/etc/apache2/apache2.conf" "/tmp/'), True, True, None,
         ).next_call().with_matching_args(
-            'chown username "/tmp/%s"' % name, True, True, None,
+            fudge_arg.startswith('chown username "/tmp/'), True, True, None,
         ).next_call().with_matching_args(
-            'chmod 400 "/tmp/%s"' % name, True, True, None,
+            fudge_arg.startswith('chmod 400 "/tmp/'), True, True, None,
         ).next_call().with_matching_args(
-            'rm -f "/tmp/%s"' % name, True, True, None,
+            fudge_arg.startswith('rm -f "/tmp/'), True, True, None,
         )
         fake_get = Fake('get', callable=True, expect_call=True).with_args(
-            '/tmp/%s' % name, fudge_arg.any_value())
+            fudge_arg.startswith('/tmp/'), fudge_arg.any_value())
 
         with hide('everything'):
             with patched_context('fabric.operations', '_run_command', fake_run):
@@ -852,7 +871,7 @@ class TestFileTransfers(FabricTest):
         put()'s local_path arg should take file-like objects too
         """
         local = self.path('whatever')
-        fake_file = StringIO()
+        fake_file = six.StringIO()
         fake_file.write("testing file-like objects in put()")
         pointer = fake_file.tell()
         target = '/new_file.txt'
@@ -891,7 +910,7 @@ class TestFileTransfers(FabricTest):
         """
         f = 'uploaded.txt'
         with hide('everything'):
-            eq_(put(StringIO('contents'), f), [f])
+            eq_(put(six.StringIO('contents'), f), [f])
 
     @server()
     def test_put_return_value_failed_attribute(self):
@@ -899,7 +918,7 @@ class TestFileTransfers(FabricTest):
         put()'s return value should indicate any paths which failed to upload.
         """
         with settings(hide('everything'), warn_only=True):
-            f = StringIO('contents')
+            f = six.StringIO('contents')
             retval = put(f, '/nonexistent/directory/structure')
         eq_(["<StringIO>"], retval.failed)
         assert not retval.succeeded
@@ -938,12 +957,10 @@ class TestFileTransfers(FabricTest):
         """
         put(use_sudo=True) works by uploading a the `local_path` to a temporary path and then moving it to a `remote_path`
         """
-        # the sha1 hash is the unique filename of the file being downloaded. sha1(<filename>)
         fake_run = Fake('_run_command', callable=True, expect_call=True).with_matching_args(
-            'mv "7c91837ec0b3570264a325df6b7ef949ee22bc56" "/foobar.txt"', True, True, None,
+            fudge_arg.startswith('mv "'), True, True, None,
         )
-        fake_put = Fake('put', callable=True, expect_call=True).with_args(fudge_arg.any_value(),
-                                                                          '7c91837ec0b3570264a325df6b7ef949ee22bc56')
+        fake_put = Fake('put', callable=True, expect_call=True)
 
         local_path = self.mkfile('foobar.txt', "baz")
         with hide('everything'):
@@ -961,10 +978,9 @@ class TestFileTransfers(FabricTest):
         """
         # the sha1 hash is the unique filename of the file being downloaded. sha1(<filename>)
         fake_run = Fake('_run_command', callable=True, expect_call=True).with_matching_args(
-            'mv "/tmp/7c91837ec0b3570264a325df6b7ef949ee22bc56" "/foobar.txt"', True, True, None,
+            fudge_arg.startswith('mv "'), True, True, None,
         )
-        fake_put = Fake('put', callable=True, expect_call=True).with_args(fudge_arg.any_value(),
-                                                                          '/tmp/7c91837ec0b3570264a325df6b7ef949ee22bc56')
+        fake_put = Fake('put', callable=True, expect_call=True)
 
         local_path = self.mkfile('foobar.txt', "baz")
         with hide('everything'):
@@ -1055,7 +1071,7 @@ class TestFileTransfers(FabricTest):
     @server()
     @mock_streams('stdout')
     def test_stringio_without_name(self):
-        file_obj = StringIO(u'test data')
+        file_obj = six.StringIO(u'test data')
         put(file_obj, '/')
         assert re.search('<file obj>', sys.stdout.getvalue())
 
@@ -1063,7 +1079,7 @@ class TestFileTransfers(FabricTest):
     @mock_streams('stdout')
     def test_stringio_with_name(self):
         """If a file object (StringIO) has a name attribute, use that in output"""
-        file_obj = StringIO(u'test data')
+        file_obj = six.StringIO(u'test data')
         file_obj.name = 'Test StringIO Object'
         put(file_obj, '/')
         assert re.search(file_obj.name, sys.stdout.getvalue())
