@@ -1,9 +1,8 @@
 from __future__ import with_statement
 
-from contextlib import contextmanager
 from fudge import Fake, patched_context, with_fakes
 import unittest
-from nose.tools import eq_, raises, ok_
+from nose.tools import raises, ok_
 import random
 import sys
 
@@ -11,10 +10,10 @@ import fabric
 from fabric.tasks import WrappedCallableTask, execute, Task, get_task_details
 from fabric.main import display_command
 from fabric.api import run, env, settings, hosts, roles, hide, parallel, task, runs_once, serial
-from fabric.network import from_dict
 from fabric.exceptions import NetworkError
 
-from utils import eq_, FabricTest, aborts, mock_streams, support
+from mock_streams import mock_streams
+from utils import eq_, FabricTest, aborts, support
 from server import server
 
 
@@ -33,7 +32,7 @@ class TestWrappedCallableTask(unittest.TestCase):
 
         def foo(): pass
         try:
-            task = WrappedCallableTask(foo, *args)
+            WrappedCallableTask(foo, *args)
         except TypeError:
             msg = "__init__ raised a TypeError, meaning args weren't handled"
             self.fail(msg)
@@ -44,7 +43,7 @@ class TestWrappedCallableTask(unittest.TestCase):
 
         def foo(): pass
         try:
-            task = WrappedCallableTask(foo, **kwargs)
+            WrappedCallableTask(foo, **kwargs)
         except TypeError:
             self.fail(
                 "__init__ raised a TypeError, meaning kwargs weren't handled")
@@ -52,12 +51,12 @@ class TestWrappedCallableTask(unittest.TestCase):
     def test_allows_any_number_of_args(self):
         args = [i for i in range(random.randint(0, 10))]
         def foo(): pass
-        task = WrappedCallableTask(foo, *args)
+        WrappedCallableTask(foo, *args)
 
     def test_allows_any_number_of_kwargs(self):
         kwargs = dict([("key%d" % i, i) for i in range(random.randint(0, 10))])
         def foo(): pass
-        task = WrappedCallableTask(foo, **kwargs)
+        WrappedCallableTask(foo, **kwargs)
 
     def test_run_is_wrapped_callable(self):
         def foo(): pass
